import Log from '@midefos/utils/src/Log.js';
import LogFmt from '@midefos/utils/src/LogFmt.js';

export default class GeneralController {

  static init() {
    // Override by children
    // this._init(YourRouter);
  }

  static _initPublicApi(router) {
    // Override by children.
  }

  static _initInternal(router) {
    // Override by children.
  }

  // Override by children.
  static _init(router) {
    this._initInternal(router);
    this._initPublicApi(router);
  }
  static _get(router, route, ...middlewares) {
    router.get(route, ...middlewares);
    this._log('GET', route, middlewares);
  }

  static _post(router, route, ...middlewares) {
    router.post(route, ...middlewares);
    this._log('POST', route, middlewares);
  }

  static _put(router, route, ...middlewares) {
    router.put(route, ...middlewares);
    this._log('PUT', route, middlewares);
  }

  static _patch(router, route, ...middlewares) {
    router.patch(route, ...middlewares);
    this._log('PATCH', route, middlewares);
  }

  static _delete(router, route, ...middlewares) {
    router.delete(route, ...middlewares);
    this._log('DELETE', route, middlewares);
  }

  static _log(method, route, middlewares) {
    Log.info(LogFmt.msg("Router listening").tag("router")
      .string("method", method).string("route", route).number("total_middlewares", middlewares.length)
      .log()
    );
  }

}
