import LogFmt from "./LogFmt.js";
import LogFormat from "./LogFormat.js";
import LogLevel from "./LogLevel.js";

export default class Log {

  static LOG_LEVEL;
  static LOG_FORMAT;

  static level(level = LogLevel.DEFAULT_LEVEL) {
    this.LOG_LEVEL = level;
    return this;
  }

  static format(format = LogFormat.DEFAULT_FORMAT) {
    this.LOG_FORMAT = format;
    return this;
  }

  static error(log, error = null) {
    this._initIfNeccesary();
    this._toStdErr(this._createCompleteLog(log, LogLevel.ERROR), error);
  }

  static warn(log, data = null) {
    this._initIfNeccesary();
    this._toStdErr(this._createCompleteLog(log, LogLevel.WARN), data);
  }

  static info(log, data = null) {
    this.log(log, data);
  }

  static log(log, data = null) {
    this._initIfNeccesary();
    if (this.LOG_LEVEL.shouldLogInfo()) {
      this._toStdOut(this._createCompleteLog(log, LogLevel.INFO), data);
    }
  }

  static measure(log, data = null) {
    this.debug(log, data);
  }

  static debug(log, data = null) {
    this._initIfNeccesary();
    if (this.LOG_LEVEL.shouldLogDebug()) {
      this._toStdOut(this._createCompleteLog(log, LogLevel.DEBUG), data);
    }
  }

  static trace(log, data = null) {
    this._initIfNeccesary();
    if (this.LOG_LEVEL.shouldLogTrace()) {
      this._toStdOut(this._createCompleteLog(log, LogLevel.TRACE), data);
    }
  }

  static _initIfNeccesary() {
    if (!this.LOG_LEVEL) {
      this.LOG_LEVEL = LogLevel.fromEnv();
    }
    if (!this.LOG_FORMAT) {
      this.LOG_FORMAT = LogFormat.DEFAULT_FORMAT;
    }
  }

  static _createCompleteLog(log, level = LogLevel.DEFAULT_LEVEL) {
    if (this.LOG_FORMAT.isLogfmt()
      && LogFmt.isStringLogFmt(log)) {
      return LogFmt.addLevel(log, level);
    }
    return `[${level.toString().toUpperCase()}] - ${log}`;
  }

  static _toStdOut(log, data) {
    this._log(console.log, log, data);
  }

  static _toStdErr(log, data) {
    this._log(console.error, log, data);
  }

  static _log(logger, log, data) {
    logger(log);
    if (data) {
      logger(data);
    }
  }

}
