import Environment from "./Environment.js";

class LogLevelType {
  static ERROR = 'ERROR';
  static WARN = 'WARN';
  static INFO = 'INFO';
  static DEBUG = 'DEBUG';
  static TRACE = 'TRACE';
}

export default class LogLevel {

  static ERROR = new LogLevel(LogLevelType.ERROR);
  static WARN = new LogLevel(LogLevelType.WARN);
  static INFO = new LogLevel(LogLevelType.INFO);
  static DEBUG = new LogLevel(LogLevelType.DEBUG);
  static TRACE = new LogLevel(LogLevelType.TRACE);

  static DEFAULT_LEVEL = this.INFO;

  static fromEnv() {
    const envLevel = Environment.LOG_LEVEL();
    if (envLevel) {
      return this.new(envLevel.toUpperCase());
    }
    const oldDebug = Environment.DEBUG();
    if (oldDebug) {
      return this.DEBUG;
    }
    return this.INFO;
  }

  static new(level = LogLevelType.INFO) {
    const logLevel = new LogLevel();
    logLevel.level = level;
    return logLevel;
  }

  level;

  constructor(level) {
    this.level = level;
  }

  shouldLogError() {
    return true;
  }

  shouldLogInfo() {
    return this.isInfo()
      || this.shouldLogDebug();
  }

  shouldLogDebug() {
    return this.isDebug()
      || this.shouldLogTrace();
  }

  shouldLogTrace() {
    return this.isTrace();
  }

  isError() {
    return this.level == LogLevelType.ERROR;
  }

  isError() {
    return this.level == LogLevelType.WARN;
  }

  isInfo() {
    return this.level == LogLevelType.INFO;
  }

  isDebug() {
    return this.level == LogLevelType.DEBUG;
  }

  isTrace() {
    return this.level == LogLevelType.TRACE;
  }

  toString() {
    return this.level;
  }

}
