import MathUtil from './MathUtil.js';
import StringReplacer from './StringReplacer.js';

export default class NumberExtractor {

  static NUMBER_REGEX = /\d*[,|.]?\d+[,|.]\d+|\d+/;
  static PLACEHOLDER_TEXT = `DO_NOT_REPLACE`;

  static fromString(string) {
    const regexValues = this.NUMBER_REGEX.exec(string);
    if (!regexValues) return null;

    let numberString = regexValues[0];
    numberString = this.correctDecimalSeparators(numberString);
    let number = Number(numberString.trim());

    const multiplier = this.extractMultiplier(string);
    number = MathUtil.multiplyNumber(number, multiplier);

    return number;
  }

  static extractMultiplier(string) {
    const regexValues = this.NUMBER_REGEX.exec(string);
    if (!regexValues) return null;

    const numberString = regexValues[0];
    const startIndex = string.indexOf(numberString) + numberString.length;
    const multiplier = string.substr(startIndex, 1);
    if (multiplier === 'K' || multiplier === 'k') {
      return multiplier;
    }
    return null;
  }

  static correctDecimalSeparators(numberString) {
    if (!numberString.includes(',')) {
      const placeholder = this.PLACEHOLDER_TEXT;
      numberString = StringReplacer.replaceIndexBy(numberString, numberString.lastIndexOf(`.`), placeholder);
      numberString = numberString.replaceAll(`.`, ``);

      const lastNumbers = numberString.substr(numberString.indexOf(placeholder) + placeholder.length);
      if (lastNumbers.length === 3) {
        numberString = numberString.replace(placeholder, ``);
        return numberString;
      }
      numberString = numberString.replace(placeholder, `.`);
      return numberString;
    };

    numberString = numberString.replaceAll(`.`, `,`);
    numberString = StringReplacer.replaceIndexBy(numberString, numberString.lastIndexOf(`,`), `.`);
    numberString = numberString.replaceAll(`,`, '');
    return numberString;
  }

}
