import Log from './Log.js';
import LogFmt from './LogFmt.js';

export default class Responder {

  static OK(res) {
    this.JSON(res, { status: 'OK' });
  }

  static UNAUTHORIZED(res) {
    this.JSON(res, { status: 'UNAUTHORIZED' }, 403);
  }

  static ERROR(res, message = 'INTERNAL SERVER ERRROR', statusCode = 500) {
    this.JSON(res, { error: message }, statusCode);
  }

  static JSON(res, object, statusCode = 200) {
    try {
      const jsonObject = JSON.stringify(object);
      res.writeHead(statusCode, { 'Content-Type': 'application/json' });
      res.end(jsonObject);
    } catch (error) {
      Log.error(LogFmt.name('responder').string('type', 'json-parse'), error);
      this.ERROR(res, 'Parsing JSON for response'.toUpperCase());
    }
  }

  static HTML(res, html, statusCode = 200) {
    res.writeHead(statusCode, { 'Content-Type': 'text/html' });
    res.end(html);
  }

  static TEXT(res, text, statusCode = 200) {
    res.writeHead(statusCode, { 'Content-Type': 'text/plain' });
    res.end(text);
  }

  static REDIRECT(res, url, statusCode = 302) {
    res.writeHead(statusCode, { Location: url });
    res.end();
  }

  static XML(res, xml, statusCode = 200) {
    res.writeHead(statusCode, { 'Content-Type': 'application/xml' });
    res.end(xml);
  }

}
